/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.flink.kubernetes.highavailability;

import org.apache.flink.kubernetes.configuration.KubernetesLeaderElectionConfiguration;
import org.apache.flink.kubernetes.kubeclient.FlinkKubeClient;
import org.apache.flink.kubernetes.kubeclient.KubernetesConfigMapSharedWatcher;
import org.apache.flink.runtime.leaderelection.LeaderElectionDriver;
import org.apache.flink.runtime.leaderelection.LeaderElectionDriverFactory;
import org.apache.flink.util.Preconditions;

import java.util.concurrent.Executor;

/** Factory that instantiates a {@link KubernetesLeaderElectionDriver}. */
public class KubernetesLeaderElectionDriverFactory implements LeaderElectionDriverFactory {
    private final FlinkKubeClient kubeClient;

    private final KubernetesLeaderElectionConfiguration kubernetesLeaderElectionConfiguration;

    private final KubernetesConfigMapSharedWatcher configMapSharedWatcher;

    private final Executor watchExecutor;

    public KubernetesLeaderElectionDriverFactory(
            FlinkKubeClient kubeClient,
            KubernetesLeaderElectionConfiguration kubernetesLeaderElectionConfiguration,
            KubernetesConfigMapSharedWatcher configMapSharedWatcher,
            Executor watchExecutor) {
        this.kubeClient = Preconditions.checkNotNull(kubeClient);
        this.kubernetesLeaderElectionConfiguration =
                Preconditions.checkNotNull(kubernetesLeaderElectionConfiguration);
        this.configMapSharedWatcher = Preconditions.checkNotNull(configMapSharedWatcher);
        this.watchExecutor = Preconditions.checkNotNull(watchExecutor);
    }

    @Override
    public KubernetesLeaderElectionDriver create(
            LeaderElectionDriver.Listener leaderElectionListener) throws Exception {
        return new KubernetesLeaderElectionDriver(
                kubernetesLeaderElectionConfiguration,
                kubeClient,
                leaderElectionListener,
                configMapSharedWatcher,
                watchExecutor);
    }
}
