/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.flink.table.operations;

import org.apache.flink.annotation.Internal;

/**
 * An {@link Operation} that describes the catalog/database switch statements, e.g. USE CATALOG or
 * USE [catalogName.]dataBaseName.
 *
 * <p>Different sub operations can represent their special meanings. For example, a use catalog
 * operation means switching current catalog to another, while use database operation means
 * switching current database.
 */
@Internal
public interface UseOperation extends Operation, ExecutableOperation {}
